﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using PostSharp.Extensibility;
using PostSharp.Constraints;
using PostSharp.Reflection;

namespace $rootnamespace$
{
    // This is an example of an architectural constraint that verifies that an interface can only be implemented
    // by classes that derive from a given class or interface.
   
    // TODO: Use MulticastAttributeUsage to specify to which elements of code the constraint can be applied.
    [MulticastAttributeUsage(MulticastTargets.Interface)]
    public class $fileinputname$ : ReferentialConstraint
    {
        Type requiredBaseType;

        // A constraint is a custom attribute, and this is the constructor of your custom attribute. You can also add public fields 
        // and public properties.
        public $fileinputname$(Type requiredBaseType)
        {
            this.requiredBaseType = requiredBaseType;
        }

        
        public override bool ValidateConstraint(object target)
        {
            // This method is called once at build time for every element of code to which the constraint has been added.

            Type interfaceType = (Type)target;

            // TODO: Validate that the constraint custom attribute is valid and that is has been added to a relevant
            // element of code. 

            if (this.requiredBaseType == null)
            {
                Message.Write(interfaceType, SeverityType.Error,
                              "MY001", "Error in {0} applied on {1}: constructor argument cannot be null.", this.GetType().Name, interfaceType);
                return false;
            }

            if (this.requiredBaseType.IsAssignableFrom(interfaceType))
            {
                Message.Write(interfaceType, SeverityType.Error,
                              "MY002", "Error in {0} applied on {1}: {2} already derives from {1}.", this.GetType().Name, interfaceType, this.requiredBaseType);
                return false;
            }


            // If you return false, the project build will fail and ValidateCode

            return true;
        }

        public override void ValidateCode(object target, Assembly assembly)
        {

            // This method is called once at build time for every element of code to which the constraint has been added,
            // and for each assembly referencing this element of code.

            Type interfaceType = (Type)target;

            // TODO: Validate usage of the element of code to which the constraint has been added.

            foreach (TypeInheritanceCodeReference usage in ReflectionSearch.GetDerivedTypes(interfaceType))
            {
                if (usage.DerivedType.IsClass && !this.requiredBaseType.IsAssignableFrom(usage.DerivedType))
                {
                    Message.Write(usage.DerivedType, SeverityType.Error,
                            "MY003", "{0} cannot implement interface {1} because it is not derived from {2}.", usage.DerivedType,
                            interfaceType, this.requiredBaseType);
                }
            }

        }
    }
}
