﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Runtime.Serialization;
using PostSharp.Aspects;
using PostSharp.Extensibility;
using PostSharp.Reflection;

namespace $rootnamespace$
{
    // This is an example of aspect provider that automatically adds [DataContract] and [DataMember].
    
    // We set up multicast inheritance so  the aspect is automatically added to children types. This is optional.
    [MulticastAttributeUsage(Inheritance = MulticastInheritance.Strict)]

    // Since we want the aspect to be applied on types, we derive our class from TypeLevelAspect.
    // If you want to have a project-wide aspect provider, derive the class from AssemblyLevelAspect.
    public class $fileinputname$ : TypeLevelAspect, IAspectProvider
    {
        // This method is called at build time and should just provide other aspects.

        public IEnumerable<AspectInstance> ProvideAspects(object targetElement)
        {
            // TODO: Replace with your own implementation. You can return instances
            // of any aspect, not only CustomAttributeIntroductionAspect.

            Type targetType = (Type)targetElement;

            CustomAttributeIntroductionAspect introduceDataContractAspect =
                new CustomAttributeIntroductionAspect(
                    new ObjectConstruction(typeof(DataContractAttribute).GetConstructor(Type.EmptyTypes)));

            CustomAttributeIntroductionAspect introduceDataMemberAspect =
                new CustomAttributeIntroductionAspect(
                    new ObjectConstruction(typeof(DataMemberAttribute).GetConstructor(Type.EmptyTypes)));


            // Add the DataContract attribute to the type.
            yield return new AspectInstance(targetType, introduceDataContractAspect);

            // Add a DataMember attribute to every relevant property.
            foreach (PropertyInfo property in
                targetType.GetProperties(BindingFlags.Public | BindingFlags.DeclaredOnly | BindingFlags.Instance))
            {
                if (property.CanWrite)
                    yield return new AspectInstance(property, introduceDataMemberAspect);
            }
        }
    }
    
}